@@ -3,7 +3,7 @@
  *
  *  Copyright (C) 1994  Scott D. Heavner
  *
- *  $Id: lde.h,v 1.32 2002/01/12 01:51:57 scottheavner Exp $
+ *  $Id: lde.h,v 1.39 2003/12/03 18:07:20 scottheavner Exp $
  */
 
 #ifndef LDE_H
@@ -17,6 +17,11 @@
 #define O_BINARY 0
 #endif
 
+#if HAVE_BZERO
+#else
+#define bzero(a,b) memset((a),0,(b))
+#endif
+
 extern char *program_name;
 extern char *device_name;
 
@@ -27,7 +32,7 @@ extern int check_root(void);
 extern void (*lde_warn)(char *fmt, ...);
 extern int  (*mgetch)(void);
 
-#define MAX_BLOCK_SIZE    4096  /* must be at least EXT2_MAX_BLOCK_SIZE or whatever the biggest FS we are using */
+#define MAX_BLOCK_SIZE    8224  /* must be at least EXT2_MAX_BLOCK_SIZE or whatever the biggest FS we are using */
 
 #define INODE_BLKS 15 /* EXT2_N_BLOCKS or higher -- can't use EXT2 references after
 		       * mulitiple architecture support was added to ext2.
@@ -39,6 +44,13 @@ struct _lde_buffer {
 };
 typedef struct _lde_buffer lde_buffer;
 
+struct _lde_dirent {
+  unsigned long inode_nr;
+  char *name;
+  int isdel;
+};
+typedef struct _lde_dirent lde_dirent;
+
 #define EMPTY_LDE_BUFFER { 0UL, NULL }
 
 /*
@@ -210,7 +222,7 @@ struct {
      -- inode tables, block bitmaps, etc, etc */
   int (*is_system_block)(unsigned long nr);
   /* Get dir name and inode number */
-  char* (*dir_entry)(int i, lde_buffer *block_buffer, unsigned long *inode_nr);
+  int (*dir_entry)(int i, lde_buffer *block_buffer, lde_dirent *d);
   /* Copies the FS specific inode into a generic inode structure */
   struct Generic_Inode* (*read_inode)(unsigned long inode_nr);
   /* Copies the generic inode to a FS specific one, then write it to disk */
@@ -222,7 +234,7 @@ struct {
 } FS_cmd;
 
 /* Flags */
-volatile struct _lde_flags {
+struct _lde_flags {
   int search_all;
   int quiet;
   int write_ok;
@@ -235,8 +247,12 @@ volatile struct _lde_flags {
   int mounted;
   int blanked_indirects;  /* Set for Linux 2.0.* to work around blanked indirect blocks */
   int logtofile;
-} lde_flags;
+  int nosymbolic_guid;
+  int byteswap;
+  int displaydeleted;
+};
 
+extern volatile struct _lde_flags lde_flags;
 extern struct sbinfo *sb;
 extern struct fs_constants *fsc;
 
@@ -271,4 +287,36 @@ extern int current_error;
 /* #define mark_zone(x) (setbit(zone_map,(x)-sb->first_dat_zone+1),changed=1) */
 /* #define unmark_zone(x) (clrbit(zone_map,(x)-sb->first_data_zone+1),changed=1) */
 
+
+/* Byte swapping routines, taken from linux kernel v2.2.19 */
+#if HAVE_ASM_TYPES_H
+#include <asm/types.h>
+#endif
+#define ___ldeswab16(x) \
+        ((__u16)( \
+                (((__u16)(x) & (__u16)0x00ffU) << 8) | \
+                (((__u16)(x) & (__u16)0xff00U) >> 8) ))
+
+#define ___ldeswab32(x) \
+        ((__u32)( \
+                (((__u32)(x) & (__u32)0x000000ffUL) << 24) | \
+                (((__u32)(x) & (__u32)0x0000ff00UL) <<  8) | \
+                (((__u32)(x) & (__u32)0x00ff0000UL) >>  8) | \
+                (((__u32)(x) & (__u32)0xff000000UL) >> 24) ))
+
+#define ___ldeswab64(x) \
+        ((__u64)( \
+                (__u64)(((__u64)(x) & (__u64)0x00000000000000ffULL) << 56) | \
+                (__u64)(((__u64)(x) & (__u64)0x000000000000ff00ULL) << 40) | \
+                (__u64)(((__u64)(x) & (__u64)0x0000000000ff0000ULL) << 24) | \
+                (__u64)(((__u64)(x) & (__u64)0x00000000ff000000ULL) <<  8) | \
+                (__u64)(((__u64)(x) & (__u64)0x000000ff00000000ULL) >>  8) | \
+                (__u64)(((__u64)(x) & (__u64)0x0000ff0000000000ULL) >> 24) | \
+                (__u64)(((__u64)(x) & (__u64)0x00ff000000000000ULL) >> 40) | \
+                (__u64)(((__u64)(x) & (__u64)0xff00000000000000ULL) >> 56) ))
+
+#define ldeswab16(x) ((lde_flags.byteswap)?(___ldeswab16(x)):(x))
+#define ldeswab32(x) ((lde_flags.byteswap)?(___ldeswab32(x)):(x))
+#define ldeswab64(x) ((lde_flags.byteswap)?(___ldeswab64(x)):(x))
+
 #endif /* LDE_H */
