@@ -3,22 +3,25 @@
  *
  *  Copyright (C) 1994  Scott D. Heavner
  *
- *  $Id: main_lde.c,v 1.42 2002/01/12 04:56:31 scottheavner Exp $
+ *  $Id: main_lde.c,v 1.53 2003/12/07 05:55:47 scottheavner Exp $
  */
 
+#include <signal.h>
+#if HAVE_GETOPT_H
+#include <getopt.h>
+#endif
 #if HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
+#if HAVE_SYS_FCNTL_H
+#include <sys/fcntl.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_GETOPT_H
-#include <getopt.h>
-#endif
-#include <signal.h>
 #include <time.h>
 
 #include <sys/stat.h>
@@ -33,8 +36,8 @@
 #  include "nc_lde.h"
 #endif
 
-#ifndef volatile
-#define volatile
+#ifndef __sighandler_t
+#define __sighandler_t void *
 #endif
 
 /* Some internal structures */
@@ -58,7 +61,7 @@ struct _main_opts {
 
 struct _search_types {
   char *name;
-  char *string;
+  unsigned char *string;
   int length; 
   int offset;
 };
@@ -80,7 +83,7 @@ char *badblocks_directory = NULL;
 
 int CURR_DEVICE = 0;
 volatile struct _lde_flags lde_flags = 
-  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } ;
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } ;
 
 void (*lde_warn)(char *fmt, ...) = tty_warn;
 int  (*mgetch)(void) = tty_mgetch;
@@ -96,24 +99,26 @@ static int check_mount(char *device_name)
   struct stat statbuf;
 
   fd = open("/etc/mtab",O_RDONLY|O_BINARY);
-  fstat(fd, &statbuf);
+  if ( fd > 0 ) {
+    fstat(fd, &statbuf);
 
-  mtab = malloc(statbuf.st_size+1);
-  if (mtab==NULL) {
-    lde_warn("Out of memory reading /etc/mtab");
+    mtab = malloc(statbuf.st_size+1);
+    if (mtab==NULL) {
+      lde_warn("Out of memory reading /etc/mtab");
+      close(fd);
+      exit(-1);
+    }
+    read(fd, mtab, statbuf.st_size);
     close(fd);
-    exit(-1);
-  }
-  read(fd, mtab, statbuf.st_size);
-  close(fd);
 
-  /* Set last character to 0 (we've allocated a space for the 0) */
-  mtab[statbuf.st_size] = 0;
-  if (strstr(mtab, device_name))
-    lde_flags.mounted = 1;
-  else
-    lde_flags.mounted = 0;
-  free(mtab);
+    /* Set last character to 0 (we've allocated a space for the 0) */
+    mtab[statbuf.st_size] = 0;
+    if (strstr(mtab, device_name))
+      lde_flags.mounted = 1;
+    else
+      lde_flags.mounted = 0;
+    free(mtab);
+  }
 
   return lde_flags.mounted;
 }
@@ -151,14 +156,15 @@ void read_tables(int fs_type, unsigned long blocksize)
   lde_warn("User requested %s filesystem. Checking device . . .",lde_typedata[fs_type].name);
 
   if ( fs_type == AUTODETECT ) {
-    for ( ++fs_type ; fs_type<LAST_FSTYPE; fs_type++) {
-      if (lde_typedata[fs_type].test(super_block_buffer,1)) {
+    for ( ++fs_type ; fs_type<LAST_AUTO_FSTYPE; fs_type++) {
+      if (lde_typedata[fs_type].test &&
+	  lde_typedata[fs_type].test(super_block_buffer,1)) {
 	break;
       }
     }
   }
 
-  if ( fs_type < LAST_FSTYPE ) {
+  if ( fs_type < LAST_AUTO_FSTYPE ) {
     lde_typedata[fs_type].init(super_block_buffer);
   } else {
     lde_warn("No file system found on device");
@@ -218,8 +224,8 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
 {
   int option_index = 0, i;
   int c;
-  static char gzip_tar_type[] = { 31, 138, 8, 0 };
-  static char gzip_type[] = { 31, 139 };
+  static unsigned char gzip_tar_type[4] = { 31, 138, 8, 0 };
+  static unsigned char gzip_type[] = { 31, 139 };
   struct _search_types search_types[] = {
       { "tgz", gzip_tar_type, 4, 0 },
       { "gz", gzip_type, 2, 0},
@@ -245,11 +251,12 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
       {"quiet", 0, 0, 'q'},
       {"offset",1,0,'O'},
       {"length",1,0,'L'},
-      {"indirects",0,0,'!'},
-      {"blanked-indirects", 0, 0, '0'},
-      {"ilookup",0,0,'@'},
-      {"recoverable",0,0,'#'},
-      {"append",0,0,'%'},
+      {"indirects",0,0,'j'},
+      {"blanked-indirects", 0, 0, 'J'},
+      {"ilookup",0,0,'k'},
+      {"recoverable",0,0,'R'},
+      {"append",0,0,'A'},
+      {"nosymbolic",0,0,'y'},
       {"file",1,0,'f'},
       {"logtofile",0,0,'F'},
       {"superscan",0,0,'P'},
@@ -265,11 +272,12 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
   while (1) {
     option_index = 0;
 
+#define ALLOPTS "AaB:b:D:d:Ff:gHhI:i:JjkL:l:N:n:O:PpqrRS:s:T:t:vwX:y?"
 #if HAVE_GETOPT_LONG
-    c = getopt_long (argc, argv, "avFf:I:i:n:N:B:b:D:d:gpPqs:S:t:T:X:whH?O:L:",
+    c = getopt_long (argc, argv, ALLOPTS,
 		     long_options, &option_index);
 #else
-    c = getopt (argc, argv, "avFf:I:i:n:N:B:b:D:d:gpPqs:S:t:T:X:whH?O:L:");
+    c = getopt (argc, argv, ALLOPTS);
 #endif
 
     if (c == -1)
@@ -295,7 +303,7 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
       case 'a': /* Search disk space marked in use as well as unused */
 	lde_flags.search_all = 1;
 	break;
-      case '0': /* Linux 2.0 blanked indirect workaround */
+      case 'J': /* Linux 2.0 blanked indirect workaround */
 	lde_flags.blanked_indirects = 1;
 	break;
       case 'g': /* Search for an inode which contains the specified block */
@@ -331,7 +339,7 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
 	opts->superscan = 1;
 	opts->skiptableread = 1;
 	break;
-      case 'q': /* no audio -- well nop beeps */
+      case 'q': /* no audio -- well no beeps */
 	lde_flags.quiet = 1;
 	break;
       case 's': /* Override blocksize */
@@ -342,7 +350,7 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
 	opts->search_len = strlen(opts->search_string);
 	break;
       case 't': /* Specify the FS on the disk */
-	i = NONE;
+	i = AUTODETECT;
 	while (lde_typedata[i].name) {
 	  if (!strncmp(optarg, lde_typedata[i].name, strlen(optarg))) {
 	    opts->fs_type = i;
@@ -350,9 +358,9 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
 	  }
 	  i++;
 	}
-	if (opts->fs_type==AUTODETECT) {
+	if (!lde_typedata[i].name) {
 	  lde_warn("`%s' type not recognized.",optarg);
-	  i = NONE;
+	  i = AUTODETECT;
 	  fprintf(stderr,"Supported file systems include: ");
 	  while (lde_typedata[i].name) {
 	    fprintf(stderr,"\"%s\" ",lde_typedata[i].name);
@@ -392,7 +400,7 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
       case 'O': /* Set offset for search string */
 	opts->search_off = read_num(optarg);
 	break;
-	  case 'F': /* Log all errors/messages to /tmp/ldeerrors */
+      case 'F': /* Log all errors/messages to /tmp/ldeerrors */
 	lde_flags.logtofile = 1;
 	break;
       case 'L': /* Set length for search string */
@@ -405,20 +413,23 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
       case 'w': /* Set FS writable */
 	lde_flags.write_ok = 1;
 	break;
-      case '!': /* Search for indirect blocks. */
+      case 'j': /* Search for indirect blocks. */
 	lde_flags.indirect_search = 1;
 	opts->search_string = "";
 	opts->search_len = 0;
 	break;
-      case '@': /* Lookup inodes on search matches. */
+      case 'k': /* Lookup inodes on search matches. */
 	lde_flags.inode_lookup = 1;
 	break;
-      case '#': /* Check for recoverablilty on search matches. */
+      case 'R': /* Check for recoverablilty on search matches. */
 	lde_flags.check_recover = 1;
 	break;
-      case '%': /* Always append data when recovery file exists */
+      case 'A': /* Always append data when recovery file exists */
 	lde_flags.always_append = 1;
 	break;
+      case 'y': /* Don't do symbolic uid/gid lookups */
+        lde_flags.nosymbolic_guid = 1;
+        break;
       case 'f': /* Specify name of recovery file */
 	opts->recover_file_name = optarg;
 	break;
@@ -450,6 +461,7 @@ int main(int argc, char ** argv)
   int i, hasdata, fp;
   unsigned long nr, inode_nr;
   char *thispointer;
+  volatile long endian = 0L;
 
   struct Generic_Inode *GInode = NULL;
 
@@ -460,11 +472,20 @@ int main(int argc, char ** argv)
 
   /* Set things up to handle control-c:  just sets lde_flags.quit_now to 1 */
   sigemptyset(&sa_mask);
-  intaction.sa_handler = (void *)handle_sigint;
+  intaction.sa_handler = (__sighandler_t) handle_sigint;
   intaction.sa_mask = sa_mask;
   intaction.sa_flags = SA_RESTART;
   sigaction(SIGINT,&intaction,NULL);
 
+  /* quick test for endianness, blindly assuming sizeof(char) != sizeof(long) */
+  *(char *)&endian = 1;
+  if ( endian != 1L ) {
+    /* lde_warn("Detected big endian system, assuming fs is little endian"); */
+    lde_flags.byteswap = 1;
+  }
+
+  init_disk_cache();
+
   parse_cmdline(argc, argv, &main_opts);
 
   if (check_mount(device_name)&&!lde_flags.paranoid)
@@ -567,7 +588,7 @@ int main(int argc, char ** argv)
 	    if ((hasdata)&&(check_recover_file(GInode->i_zone, GInode->i_size))) {
 		printf("Inode 0x%lX recovery possible",nr);
 		if (fsc->inode->i_dtime) {
-                  thispointer = ctime(&(GInode->i_dtime));
+                  thispointer = ctime((time_t *)&(GInode->i_dtime));
                   thispointer[24] = 0;
 		  printf(" (deleted %24s)",thispointer);
                 }
@@ -628,7 +649,7 @@ int main(int argc, char ** argv)
     parse_grep();
     exit(0);
   } else if (main_opts.search_string!=NULL) {
-    search_fs(main_opts.search_string, main_opts.search_len, main_opts.search_off,main_opts.dump_end);
+    search_fs( (unsigned char *)main_opts.search_string, main_opts.search_len, main_opts.search_off,main_opts.dump_end);
     exit(0);
   } else if (main_opts.superscan) {
     search_for_superblocks(main_opts.fs_type);
