@@ -3,7 +3,7 @@
  *
  *  Copyright (C) 1994  Scott D. Heavner
  *
- *  $Id: msdos_fs.c,v 1.20 2001/11/26 03:10:41 scottheavner Exp $
+ *  $Id: msdos_fs.c,v 1.24 2002/05/29 05:38:32 scottheavner Exp $
  */
 
 /* 
@@ -35,7 +35,7 @@
 #endif
 
 static struct Generic_Inode *DOS_read_inode(unsigned long nr);
-static char* DOS_dir_entry(int i, lde_buffer *block_buffer, unsigned long *inode_nr);
+static int DOS_dir_entry(int i, lde_buffer *block_buffer, lde_dirent *d);
 static void DOS_sb_init(void * sb_buffer);
 static unsigned long DOS_map_inode(unsigned long ino);
 static int DOS_write_inode_NOTYET(unsigned long ino,
@@ -226,19 +226,23 @@ static struct Generic_Inode *DOS_read_inode(unsigned long nr)
   return &DOS_junk_inode;
 }
 
-static char* DOS_dir_entry(int i, lde_buffer *block_buffer, unsigned long *inode_nr)
+static int DOS_dir_entry(int i, lde_buffer *block_buffer, lde_dirent *d)
 {
   static char cname[MSDOS_NAME+2+14];
   struct msdos_dir_entry *dir;
   struct msdos_dir_slot  *slot;
+  int retval = 0;
+
+  bzero(d,sizeof(lde_dirent));
+  d->name = cname;
 
   if (i*sb->dirsize >= block_buffer->size) {
     cname[0] = 0;
   } else {
     dir = block_buffer->start+(i*sb->dirsize);
 
-    *inode_nr = (unsigned long) dir->start + (((unsigned long)dir->starthi)<<16);
-    DOS_dir_inode = *inode_nr;
+    d->inode_nr = (unsigned long) ldeswab16(dir->start) + (((unsigned long)ldeswab16(dir->starthi))<<16);
+    DOS_dir_inode = d->inode_nr;
 
     slot = (void *)dir;
     if ( (slot->attr == 0xF) && (!slot->reserved) ) {
@@ -253,7 +257,7 @@ static char* DOS_dir_entry(int i, lde_buffer *block_buffer, unsigned long *inode
       cname[12] = slot->name11_12[2];
       cname[13] = 0;
     } else {
-      memset(cname,MSDOS_NAME+2,' ');
+      memset(cname,' ',MSDOS_NAME+2);
       strncpy(cname, dir->name, 8); /* File name */
       cname[8] = '.';
       strncpy(&cname[9], dir->ext, 3);  /* File extension */
@@ -261,14 +265,14 @@ static char* DOS_dir_entry(int i, lde_buffer *block_buffer, unsigned long *inode
     }
 
     DOS_junk_inode.i_mode = S_IRUSR|S_IROTH|S_IRGRP;
-    DOS_junk_inode.i_size = dir->size;
+    DOS_junk_inode.i_size = ldeswab32(dir->size);
     if (dir->attr&ATTR_DIR) {
        DOS_junk_inode.i_mode |= S_IFDIR|S_IXOTH|S_IXUSR|S_IXGRP;
        /* nc_dir.c:get_inode_info() gets pissy if dir length == 0 */
        DOS_junk_inode.i_size = sb->zonesize*sb->blocksize;
        /* .. seems to show up as inode 0 fairly often -- bump it to 2 */
-       if (*inode_nr==0UL)
-          *inode_nr = 2UL;
+       if (d->inode_nr==0UL)
+          d->inode_nr = 2UL;
     } else {
        DOS_junk_inode.i_mode |= S_IFREG;
     }
@@ -276,11 +280,13 @@ static char* DOS_dir_entry(int i, lde_buffer *block_buffer, unsigned long *inode
        DOS_junk_inode.i_mode |= S_IFSOCK;
     if (dir->attr&ATTR_RO)
        DOS_junk_inode.i_mode |= S_IWUSR|S_IWGRP|S_IWOTH;
-    DOS_junk_inode.i_ctime = date_dos2unix(dir->ctime,dir->cdate);
-    DOS_junk_inode.i_atime = date_dos2unix(0,dir->adate);
-    DOS_junk_inode.i_mtime = date_dos2unix(dir->time,dir->date);
+    DOS_junk_inode.i_ctime = date_dos2unix(ldeswab16(dir->ctime),ldeswab16(dir->cdate));
+    DOS_junk_inode.i_atime = date_dos2unix(0,ldeswab16(dir->adate));
+    DOS_junk_inode.i_mtime = date_dos2unix(ldeswab16(dir->time),ldeswab16(dir->date));
+
+    retval = 1;
   }
-  return (cname);
+  return retval;
 }
 
 /* This is used to pull the correct block from the inode block table which
@@ -308,29 +314,34 @@ static void DOS_sb_init(void *sb_buffer)
   struct fat_boot_sector *Boot;
   Boot = sb_buffer;
 
-  sb->blocksize = (unsigned long) align_ushort(Boot->sector_size);
-  sb->nzones = (unsigned long) align_ushort(Boot->sectors);
+  sb->blocksize = (unsigned long) ldeswab16(align_ushort(Boot->sector_size));
+  sb->nzones = (unsigned long) ldeswab16(align_ushort(Boot->sectors));
   if ( !sb->nzones )
-    sb->nzones = Boot->total_sect;
+    sb->nzones = ldeswab32(Boot->total_sect);
 
   sb->last_block_size = sb->blocksize;
 
   fsc->ROOT_INODE = 2;
 
+  if (Boot->cluster_size) {
+     sb->zonesize = Boot->cluster_size;
+  } else {
+     sb->zonesize = 64;
+  }
+
   /* In order to prevent division by zeroes, set junk entries to 1 */
-  sb->ninodes = sb->nzones/Boot->cluster_size;
+  sb->ninodes = sb->nzones/sb->zonesize;  /* /Boot->cluster_size */
   sb->imap_blocks = 1;
   if (!Boot->fats)  /* Could this ever happen? */
      Boot->fats = 2;
-  if (Boot->fat_length)  /* FAT12/16 */
-     sb->zmap_blocks = Boot->fats*Boot->fat_length;
+  if (ldeswab16(Boot->fat_length))  /* FAT12/16 */
+     sb->zmap_blocks = Boot->fats*ldeswab16(Boot->fat_length);
   else {
-     sb->zmap_blocks = Boot->fats*Boot->fat32_length;
+     sb->zmap_blocks = Boot->fats*ldeswab32(Boot->fat32_length);
      fsc->INODE_SIZE = 4;
   }
-  sb->first_data_zone = Boot->reserved + sb->zmap_blocks - 2 * Boot->cluster_size;
+  sb->first_data_zone = ldeswab16(Boot->reserved) + sb->zmap_blocks - 2 * sb->zonesize; /* * Boot->cluster_size */
   sb->max_size = 1;
-  sb->zonesize = Boot->cluster_size;
   sb->magic = 0;
 
   sb->I_MAP_SLOTS = 1;
