@@ -1,9 +1,9 @@
- /*
+/*
  *  lde/nc_block.c -- The Linux Disk Editor
  *
  *  Copyright (C) 1994  Scott D. Heavner
  *
- *  $Id: nc_block.c,v 1.31 2001/11/26 00:07:23 scottheavner Exp $
+ *  $Id: nc_block.c,v 1.35 2003/12/03 18:27:32 scottheavner Exp $
  */
 
 #include <stdio.h>
@@ -60,6 +60,10 @@ static void cdump_block(unsigned long nr, bm_cursor *curs)
   const char block_not_used[10] = ":NOT:USED:";
   const char block_is_bad[10]   = ":BAD::BAD:";
 
+  /* indicate we are in block mode */
+  strcpy(ldemode,"Block     ");
+  update_header();
+
   werase(workspace);
 
   /* We want to indicate bad blocks and unused blocks */
@@ -219,6 +223,9 @@ static void update_block_help(void)
 }
 
 
+/* Make sure our memory copy of 2*curs->bps+1 blocks is updated,
+ *   and centered around our current block.  If not, re-read
+ *   necessary blocks from disk. */
 static int update_block_contents(cached_block **passed_block,
 				 unsigned long new_bnr,
 				 bm_cursor *curs, bm_irecord *iptr)
@@ -259,8 +266,8 @@ static int update_block_contents(cached_block **passed_block,
   /* Save return value for passed_block */
   *passed_block = this_block;
 
+  /* Make sure prev blocks are loaded into memory */
   cbptr = this_block;
-  /* Make sure next and prev are ok */
   for (i=1; i<=curs->bps; i++) {
     cbptr = cbptr->prev;
     if (iptr) {
@@ -280,6 +287,7 @@ static int update_block_contents(cached_block **passed_block,
     }
   }
 
+  /* Make sure next blocks are loaded into memory */
   cbptr = this_block;
   for (i=1; i<=curs->bps; i++) {
     cbptr = cbptr->next;
@@ -339,10 +347,13 @@ static int update_block_data(cached_block **passed_block,
 
   curs->eod = 0;
 
+  /* back up, cbptr=block we're interested in, which is in
+   *  center of linked list */
   cbptr = this_block;
   for (i=0; i<curs->bps; i++)
     cbptr = cbptr->prev;
 
+  /* Copy prev. blocks into cbuffer */
   for (i=0; i<curs->bps; i++) {
     if ( cbptr->size ) {
       memcpy(cbuffer,cbptr->data,cbptr->size);
@@ -352,6 +363,7 @@ static int update_block_data(cached_block **passed_block,
     cbptr = cbptr->next;
   }
 
+  /* Copy current block into cbuffer */
   cbptr = this_block;
   curs->sob = curs->sow = curs->eod;
   memcpy(cbuffer,this_block->data,this_block->size);
@@ -359,6 +371,7 @@ static int update_block_data(cached_block **passed_block,
   curs->eod += this_block->size;
   curs->eob = curs->eod;
 
+  /* Copy next blocks into cbuffer */
   for (i=0; i<curs->bps; i++) {
     cbptr = cbptr->next;
     if ( cbptr->size ) {
@@ -386,13 +399,16 @@ int block_mode(void) {
   char *HEX_PTR, *HEX_NOS = "0123456789ABCDEF", *s;
   unsigned long a, temp_ptr, search_iptr=0UL, inode_ptr[2] = { 0UL, 0UL };
   bm_flags flags = { 0, 0, 0, 0, 0, 1 };
-  bm_cursor curs = { 0, 0, 0, 0, 0, 0, 16, VERT };
+  bm_cursor curs = { 0, 0, 0, 0, 0, 0, 16, 24 };
   bm_irecord irecord = { 0 }, *irecptr=NULL;
   cached_block *this_block, *cb_buffer, *tmp_block; 
 
+  /* Need to init separately as VERT is not a compile time constant */
+  curs.wl = VERT;
+
   /* Want to allocate enough room for two screenfuls of data, 
    * compute the size of one screen full here */
-  curs.bps = (VERT*(COLS-HOFF)+(sb->blocksize-1))/sb->blocksize;
+  curs.bps = (VERT*(COLS-HOFF)+(sb->blocksize-1))/sb->blocksize + 1;
 
   /* Allocate space for buffers, abort on error */
   curs.data = malloc(sb->blocksize*(2*curs.bps+1));
@@ -483,7 +499,7 @@ has command */
 #     endif /* BETA_CODE */
     } else if (flags.edit_block) {          /* Handle edit keys in edit mode */
       if (flags.ascii_mode) {
-	if isprint(c) {
+	if (isprint(c)) {
 	  curs.data[calc_offset(&curs)] = c;
 	  flags.highlight = flags.modified = flags.dontwait = 1;
 	  c = CMD_NEXT_FIELD; /* Advance the cursor */
@@ -633,8 +649,11 @@ has command */
 	break;
 
       case CMD_PREV_SCREEN:
-	if (curs.sow - VERT*curs.rs >= 0)
+	if (curs.sow - VERT*curs.rs >= 0) {
 	  curs.sow -= VERT*curs.rs;
+	} else if (!(this_block->bnr)) {
+          curs.sow = 0;
+        }
 	if (calc_offset(&curs)<curs.sob) {
 
 	  /* It is possible we may need to go back more than one block */
@@ -650,6 +669,7 @@ has command */
 	  /* Not very robust yet: (curs.sob-c) should be a
 	   * multiple of blocksize */
 	  c = (curs.sob-c)-(curs.sob-curs.sow);
+          /* if ( c < -sb->blocksize ) lde_warn("C < blocksize - %d < %ld",c,sb->blocksize); */
 	  if (update_block_data(&this_block,tmp_block->bnr,
 				irecptr, &curs, &flags, c)) {
 	    curs.sow += VERT*curs.rs;
@@ -673,8 +693,11 @@ has command */
 	break;
 
       case CMD_PREV_SCREEN:
-	if (curs.sow - VERT*curs.rs >= 0)
+	if (curs.sow - VERT*curs.rs >= 0) {
 	  curs.sow -= VERT*curs.rs;
+	} else {
+          curs.sow = 0;
+        }
 	if ((calc_offset(&curs)<curs.sob)&&(this_block->prev->size)) {
 	  if (update_block_data(&this_block,this_block->prev->bnr,
 				irecptr, &curs, &flags,
