@@ -3,15 +3,19 @@
  *
  *  Copyright (C) 1994  Scott D. Heavner
  *
- *  $Id: nc_inode.c,v 1.23 2001/02/26 18:53:45 scottheavner Exp $
+ *  $Id: nc_inode.c,v 1.26 2003/12/07 01:35:53 scottheavner Exp $
  */
 
 #include <ctype.h>
 #include <sys/stat.h>
 #include <stdlib.h>
 #include <string.h>
+#if HAVE_PWD_H
 #include <pwd.h>
+#endif
+#if HAVE_GRP_H
 #include <grp.h>
+#endif
 #include <time.h>
 
 #include "lde.h"
@@ -123,8 +127,16 @@ static void cdump_inode_values(unsigned long nr, struct Generic_Inode *GInode, i
 {
   unsigned long imode = 0UL, j = 0UL;
   char f_mode[12];
+#if HAVE_GETPWUID
   struct passwd *NC_PASS = NULL;
+#endif
+#if HAVE_GETGRGID
   struct group *NC_GROUP = NULL;
+#endif
+
+  /* update header to show we are in INODE mode */
+  strcpy(ldemode,"Inode       ");
+  update_header();
 
   if (highlight_field&LDE_DUMP_ILABELS) {
     cdump_inode_labels();
@@ -143,15 +155,19 @@ static void cdump_inode_values(unsigned long nr, struct Generic_Inode *GInode, i
     mvwprintw(workspace,0,11,"%3d",GInode->i_links_count);
 
   if (fsc->inode->i_uid) {
+#if HAVE_GETPWUID
     if ((NC_PASS = getpwuid(GInode->i_uid))!=NULL)
       mvwprintw(workspace,0,15,"%-8s",NC_PASS->pw_name);
     else
+#endif
       mvwprintw(workspace,0,15,"%-8d",GInode->i_uid);
   }
   if (fsc->inode->i_gid) {
+#if HAVE_GETGRGID
     if ((NC_GROUP = getgrgid(GInode->i_gid))!=NULL)
       mvwprintw(workspace,0,24,"%-8s",NC_GROUP->gr_name);
     else
+#endif
       mvwprintw(workspace,0,24,"%-8d",GInode->i_gid);
   }
   if (fsc->inode->i_size)
@@ -198,7 +214,9 @@ static void cdump_inode_values(unsigned long nr, struct Generic_Inode *GInode, i
   }
 
   if (fsc->inode->i_uid) {
+#if HAVE_GETPWUID
     mvwprintw(workspace,4,10,"(%s)",(NC_PASS != NULL) ? NC_PASS->pw_name : "");
+#endif
     if (highlight_field == I_UID) {
       wattron(workspace,WHITE_ON_RED);
       park_y = 4;
@@ -209,7 +227,9 @@ static void cdump_inode_values(unsigned long nr, struct Generic_Inode *GInode, i
   }
 
   if (fsc->inode->i_gid) {
+#if HAVE_GETGRGID
     mvwprintw(workspace,4,30,"(%s)",(NC_GROUP != NULL) ? NC_GROUP->gr_name : "");
+#endif
     if (highlight_field == I_GID) {
       wattron(workspace,WHITE_ON_RED);
       park_y = 4;
@@ -400,7 +420,7 @@ void parse_edit(WINDOW *workspace, int c, int *modified, struct Generic_Inode *G
 		int highlight_field, int park_x, int park_y )
 {
   unsigned long a;
-  int result;
+  int result = 0;
   char *s;
 
 #ifndef NC_FIXED_UNGETCH
@@ -410,7 +430,14 @@ void parse_edit(WINDOW *workspace, int c, int *modified, struct Generic_Inode *G
     case I_MTIME:
     case I_CTIME:
       if ((result = ncread("Enter new time and/or date:",NULL,&s)))
-	a = (unsigned long) lde_getdate(s,NULL);
+      {
+	a = (unsigned long) lde_getdate(s);
+	if ( a == -1 )
+        {
+	  lde_warn( "Bad date - %s", s );
+          result = 0;
+        }
+      }
       break;
     default:
       result = ncread("Enter new value: ",&a,NULL);
@@ -434,12 +461,18 @@ void parse_edit(WINDOW *workspace, int c, int *modified, struct Generic_Inode *G
     wgetnstr(workspace, cinput, 10);
     nodelay(workspace,FALSE);
     noecho();
+    result = 1;
     switch(highlight_field) {
       case I_ATIME:
       case I_DTIME:
       case I_MTIME:
       case I_CTIME:
-	a = (unsigned long) lde_getdate(cinput,NULL);
+	a = (unsigned long) lde_getdate(cinput);
+        if ( a == -1 )
+	{
+           result = 0;
+	   lde_warn( "Bad date - %s", cinput );
+        }
 	break;
       default:
 	a = read_num(cinput);
@@ -448,7 +481,7 @@ void parse_edit(WINDOW *workspace, int c, int *modified, struct Generic_Inode *G
     *modified = 1;
   }
 #endif /* NC_FIXED_UNGETCH */
-  if (!lde_flags.write_ok) lde_warn("Disk not writeable, change status flags with (F)");
+  if (!lde_flags.write_ok && result ) lde_warn("Disk not writeable, change status flags with (F)");
 }
 
 
