@@ -3,7 +3,7 @@
  *
  *  Copyright (C) 1994  Scott D. Heavner
  *
- *  $Id: recover.c,v 1.36 2002/01/12 05:42:53 scottheavner Exp $
+ *  $Id: recover.c,v 1.42.2.1 2005/05/19 06:47:55 scottheavner Exp $
  */
 
 #include <stdio.h>
@@ -28,15 +28,15 @@
 unsigned long block_pointer(unsigned char *ind, 
 			    unsigned long blknr, int zone_entry_size)
 {
-  unsigned long  *lind;
-  unsigned short *sind;
+  __u32 *lind;
+  __u16 *sind;
 
   if (zone_entry_size == 2) {
-    sind = (unsigned short *) ind;
-    return sind[blknr];
+    sind = (__u16 *) ind;
+    return ldeswab16(sind[blknr]);
   } else {
-    lind = (unsigned long *) ind;
-    return lind[blknr];
+    lind = (__u32 *) ind;
+    return ldeswab32(lind[blknr]);
   }
 }
 
@@ -471,7 +471,9 @@ int recover_file(int fp,unsigned long zone_index[],unsigned long filesize)
 }
 
 /* Goes through and checks to see if the file can be recovered.  I.e. is another
- * file using blocks that were once used by this inode */
+ * file using blocks that were once used by this inode.
+ *   0 = recovery possible 
+ *   1 = not possible */
 int check_recover_file(unsigned long zone_index[],unsigned long filesize)
 {
   unsigned long nr, lookedup=0UL, checked=0UL;
@@ -585,24 +587,25 @@ unsigned long find_inode(unsigned long nr, unsigned long last_nr)
 void parse_grep(void)
 {
   unsigned long inode_nr, blknr;
+  unsigned long long offset;
   int i, miss_count=0;
 
-  while ( (i=scanf("%ld\n",&blknr))!=EOF ) {
+  while ( (i=scanf("%Ld\n",&offset))!=EOF ) {
     if (i) {
       miss_count = 0;
-      if (blknr) {
-	blknr = blknr / sb->blocksize + 1;
+      if (offset) {
+	blknr = offset / sb->blocksize + 1;
 	inode_nr = find_inode(blknr, 0UL);
-	if (inode_nr) {
+	while (inode_nr) {
 	  printf("Block 0x%lX indexed by inode 0x%lX",blknr,inode_nr);
 	  if (FS_cmd.inode_in_use(inode_nr))
 	    printf(" (This inode is marked in use)");
 	  if (FS_cmd.zone_in_use(blknr))
 	    printf(" (This zone is marked in use)");
 	  printf("\n");
+          inode_nr = find_inode(blknr, 0UL);
 	}
-	else
-	  printf("Block 0x%lX is not referenced by any inode.\n",blknr);
+        printf("Block 0x%lX has no (more) references.\n",blknr);
       }
     }
     else /* EOF doesn't seem to work */
@@ -756,7 +759,8 @@ void search_fs(unsigned char *search_string, int search_len, int search_off, uns
 
 }
 
-/* Do a regular expression search across the filesystem */
+/* Do a exact pattern search across the filesystem, 
+ *  only used in curses interface */
 int search_blocks(char *searchstring, unsigned long sbnr, unsigned long *mbnr, int *moffset) {
   int     retval=0;
 
@@ -816,8 +820,8 @@ int search_blocks(char *searchstring, unsigned long sbnr, unsigned long *mbnr, i
   
   free(buffer);
 #else
-#warning No memmem() found, block search disabled.
-  lde_warn("Block search needs memmem, try recompiling.");
+#warning No memmem() found, UI/Curses block search disabled.
+  lde_warn("UI/Curses block search needs memmem, try recompiling.");
 #endif
 
   if (retval==0)
@@ -830,13 +834,13 @@ int search_blocks(char *searchstring, unsigned long sbnr, unsigned long *mbnr, i
 int search_for_superblocks(int fs_type) {
   unsigned long sbnr = 0;
   int i;
-  char *buffer[512*500];
+  char buffer[512*500];
   size_t  bytesread;
  
   /* Want to do all searches on 512 byte boundries, in case something has gotten screwed up */
   NOFS_init(NULL,512);
 
-  if ( fs_type >= LAST_FSTYPE || fs_type < AUTODETECT ) {
+  if ( fs_type >= LAST_AUTO_FSTYPE || fs_type < AUTODETECT ) {
     lde_warn("Bad filetype specified (i.e. why would \"no\" have a superblock) . . . Aborting.");
     return -1;
   }
@@ -844,18 +848,18 @@ int search_for_superblocks(int fs_type) {
   lde_warn("Searching disk for %s superblocks . . .",(fs_type==AUTODETECT)?"any":lde_typedata[fs_type].name);
 
   for ( ; sbnr<sb->nzones; ++sbnr) {
-    bytesread = nocache_read_block(sbnr, buffer, 512);
+    bytesread = nocache_read_block(sbnr, &buffer, 512);
     if (bytesread < 0)
       break;
 
     if ( fs_type == AUTODETECT ) {
-      for ( i = AUTODETECT+1 ; i<LAST_FSTYPE; i++) {
-	if (lde_typedata[i].test(buffer,0)) {
+      for ( i = AUTODETECT+1 ; lde_typedata[i].test; i++) {
+	if (lde_typedata[i].test(&buffer,0)) {
 	  lde_warn("Found %s superblock at 0x%lx",lde_typedata[i].name,sbnr);
 	}
       }
     } else {
-      if (lde_typedata[fs_type].test(buffer,0)) {
+      if (lde_typedata[fs_type].test(&buffer,0)) {
 	lde_warn("Found %s superblock at 0x%lx",lde_typedata[fs_type].name,sbnr);
       }
     }
