@@ -3,7 +3,7 @@
  *
  *  Copyright (C) 1994  Scott D. Heavner
  *
- *  $Id: tty_lde.c,v 1.28 2002/01/11 18:30:41 scottheavner Exp $
+ *  $Id: tty_lde.c,v 1.34 2003/12/06 08:01:46 scottheavner Exp $
  */
 
 #include <stdio.h>
@@ -16,10 +16,19 @@
 #if HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
+#if HAVE_SYS_FCNTL_H
+#include <sys/fcntl.h>
+#endif
 #include <time.h>
+#if HAVE_PWD_H
 #include <pwd.h>
+#endif
+#if HAVE_GRP_H
 #include <grp.h>
+#endif
+#if HAVE_ERRNO_H
 #include <errno.h>
+#endif
 #include <sys/stat.h>
 #include <time.h>
 
@@ -145,6 +154,14 @@ unsigned long lde_seek_block(unsigned long block_nr)
   if (llseek(CURR_DEVICE, dbnr, SEEK_SET)==dbnr)
     return block_nr;
 
+#else
+#if ( SIZEOF_OFF_T > SIZEOF_UNSIGNED_LONG )
+
+  off_t dbnr = (off_t)block_nr * sb->blocksize;
+
+  if (lseek(CURR_DEVICE, dbnr, SEEK_SET)==dbnr)
+    return block_nr;
+
 #else
 #define MAX_OFF_T (~(1L << (sizeof(off_t)*8-1)))
 #warning System does not have llseek() or lseek64(), using slow lookups for blocks > 2GB
@@ -174,7 +191,8 @@ unsigned long lde_seek_block(unsigned long block_nr)
 
   if (lseek(CURR_DEVICE, (off_t)b, whence)==b)
     return block_nr;
-#endif
+#endif /* off_t > ul */
+#endif /* HAVE_LLSEEK */
 #endif /* HAVE_LSEEK64 */
 
   lde_warn("lde_seek_block: seek failed, errno=%d",errno);
@@ -227,15 +245,39 @@ size_t nocache_read_block (unsigned long block_nr, void *dest,
   return act_size;
 }
 
+static cached_block diskcache[LDE_DISKCACHE];
+static cached_block *diskcachestart = diskcache;
+
+/* Assign pointers to link list, add bogus values to block number and size */
+void init_disk_cache () {
+  int i;
+  for (i=1; i<(LDE_DISKCACHE-1); i++) {
+    diskcache[i].prev = &diskcache[i-1];
+    diskcache[i].next = &diskcache[i+1];
+    diskcache[i].bnr = -1;
+    diskcache[i].size = 0;
+  }
+
+  diskcache[0].prev = &diskcache[LDE_DISKCACHE-1];
+  diskcache[0].next = &diskcache[1];
+  diskcache[0].bnr = -1;
+  diskcache[0].size = 0;
+
+  diskcache[LDE_DISKCACHE-1].prev = &diskcache[LDE_DISKCACHE-2];
+  diskcache[LDE_DISKCACHE-1].next = &diskcache[0];
+  diskcache[LDE_DISKCACHE-1].bnr = -1;
+  diskcache[LDE_DISKCACHE-1].size = 0;
+}
+
+
 /* Whopping cache of one block returns pointer to a cache_block structure
  *  The start of this struct is a buffer so it is possible to access the
  *  data buffer contents by referring to the structs address. */
 void * cache_read_block (unsigned long block_nr, void *dest, int force)
 {
-  static cached_block cache;
-  static unsigned long cache_block_nr = -1L; /* set to an outrageous number */
+  cached_block *bp, *cp;
+  int i;
 
-  cached_block *bp = &cache;
   
   /* User has specified another buffer */
   if (force&NEVER_CACHE) {
@@ -245,19 +287,41 @@ void * cache_read_block (unsigned long block_nr, void *dest, int force)
       return NULL;
     }
     bp = dest;
+  } else {
+    /* Search cache, see if we have a copy of this block */
+    bp = diskcachestart;
+    for (i=0; i<LDE_DISKCACHE; i++) {
+      if (bp->bnr == block_nr) {
+	break;
+      }
+      bp = bp->next;
+    }
+
+    /* Move this block to the top of the chain */
+    if ( bp != diskcachestart ) {
+      /* pull this block out of the chain */
+      bp->prev->next = bp->next;
+
+      /* jam it back at the top of the linked list */
+      bp->next = diskcachestart->next;
+      bp->prev = diskcachestart;
+      diskcachestart->next = bp;
+
+      /* save as new block in cache chain */
+      diskcachestart = bp;
+    }
   }
     
-  if ((force&FORCE_READ)||(block_nr != cache_block_nr)) {
+  if ((force&FORCE_READ)||(block_nr != bp->bnr)) {
     int read_size;
 
-    cache_block_nr = block_nr;
-    memset(bp->data,0,sb->blocksize);
+    bzero(bp->data,sb->blocksize);
 
     /* Lookup size of block (if it's the last block in a file,
      * it may be less than the blocksize of the device */
     read_size = lookup_blocksize(block_nr);
 
-    bp->size = nocache_read_block(cache_block_nr, bp->data, read_size);
+    bp->size = nocache_read_block(block_nr, bp->data, read_size);
     if ( bp->size == -1 ) {
 	bp->size = read_size;  /* Lousy handling */
 	memset(bp->data,'!',bp->size);
@@ -270,10 +334,10 @@ void * cache_read_block (unsigned long block_nr, void *dest, int force)
  
   /* Need to copy data to dest? */
   if ((dest)&&(bp!=dest)) {
-    bp = dest;
-    memcpy(bp,&cache,cache.size);
-    bp->bnr = cache.bnr;
-    bp->size = cache.size;
+    cp = dest;
+    memcpy(cp,bp,bp->size);
+    cp->bnr = bp->bnr;
+    cp->size = bp->size;
   }
 
   return bp;
@@ -354,8 +418,12 @@ void dump_inode(unsigned long nr)
   int j;
   char f_mode[12];
   struct Generic_Inode *GInode;
+#if HAVE_GETPWUID
   struct passwd *NC_PASS = NULL;
+#endif
+#if HAVE_GETGRGID
   struct group *NC_GROUP = NULL;
+#endif
 
   GInode = FS_cmd.read_inode(nr);
 
@@ -375,18 +443,22 @@ void dump_inode(unsigned long nr)
   printf("%10.10s	 ",f_mode);
   /* UID field */
   if (fsc->inode->i_uid) {
-    if ((NC_PASS = getpwuid(GInode->i_uid))!=NULL)
+#if HAVE_GETPWUID
+    if ( !lde_flags.nosymbolic_guid && ((NC_PASS = getpwuid(GInode->i_uid))!=NULL) )
       printf("%-8s ",NC_PASS->pw_name);
     else
+#endif
       printf("%-8d ",GInode->i_uid);
   } else {
     printf("         ");
   }
   /* GID field */
   if (fsc->inode->i_gid) {
-    if ((NC_GROUP = getgrgid(GInode->i_gid))!=NULL)
+#if HAVE_GETGRGID
+    if ( !lde_flags.nosymbolic_guid && ((NC_GROUP = getgrgid(GInode->i_gid))!=NULL) )
       printf("%-8s ",NC_GROUP->gr_name);
     else
+#endif
       printf("%-8d ",GInode->i_gid);
   } else {
     printf("         ");
@@ -425,8 +497,10 @@ void dump_inode(unsigned long nr)
   if (fsc->inode->i_uid) {
     printf("UID:                   ");
     printf("%05d",GInode->i_uid);
+#if HAVE_GETPWUID
     if (NC_PASS != NULL)
       printf(" (%s)",NC_PASS->pw_name);
+#endif
     printf("\n");
   }
 
@@ -434,8 +508,10 @@ void dump_inode(unsigned long nr)
   if (fsc->inode->i_gid) {
     printf("GID:                   ");
     printf("%05d",GInode->i_gid);
+#if HAVE_GETGRGID
     if (NC_GROUP != NULL)
       printf(" (%s)",NC_GROUP->gr_name);
+#endif
     printf("\n");
   }
 
